/*
 * file devices.c
 *
 * Routines for Pointers device processing
 *
 * Joystick and Mouse
 *
 * original idea from Chris Sharp <sharp@uk.ibm.com>
 *
 */

#define __DEVICES_C_
#include "xmame.h"
#include "devices.h"

/* local variables */
static int mouse_joy_x, mouse_joy_y;

/* 2 init routines one for creating the display and one after that, since some
   (most) init stuff needs a display */

int osd_input_initpre (void)
{
   int i;
   
   key           = NULL;
   joy_poll_func = NULL;
   mouse_joy_x   = mouse_joy_y   = 0;
   mouse_delta_x = mouse_delta_y = 0;
   mouse_b1 = mouse_b2 = mouse_b3 = 0;
   memset(joydata, 0, sizeof(struct joydata_struct) * MAX_JOY);
   for(i=0; i<MAX_JOY; i++)
   {
      joydata[i].fd = -1;
   }
   
   if (use_mouse)
      fprintf (stderr_file, "Mouse/Trakball selected.\n");
   
   return OSD_OK;
}

int osd_input_initpost (void)
{
   int i;
   
   /* setup the keys array */
#ifdef MAME_NET      
   if (netstate)
      key = global_key;
    else
#endif
      key = local_key;
   
   /* joysticks */
   switch (joytype)
   {
      case JOY_NONE:
         break;
#ifdef I386_JOYSTICK
      case JOY_I386NEW:
      case JOY_I386:
         joy_i386_init();
         break;
#endif
#ifdef LIN_FM_TOWNS
      case JOY_PAD:
         joy_pad_init ();
         break;
#endif
#ifdef X11_JOYSTICK
      case JOY_X11:
         joy_x11_init();
         break;
#endif
      default:
         fprintf (stderr_file, "Warning, unknown joytype: %d, or joytype not compiled in.\n"
            "Disabling joystick support.\n", joytype);
         joytype = JOY_NONE;
   }
   
   /* to avoid deviding by 0 and jitter of unused joysticks */
   for (i = 0; i < MAX_JOY; i++)
   {
      joydata[i].x = joydata[i].center_x;
      joydata[i].y = joydata[i].center_y;
      joydata[i].min_x = joydata[i].center_x - 10;
      joydata[i].min_y = joydata[i].center_y - 10;
      joydata[i].max_x = joydata[i].center_x + 10;
      joydata[i].max_y = joydata[i].center_y + 10;
   }

   return OSD_OK;
}

void osd_input_close (void)
{
   int i;
   
   for(i=0;i<MAX_JOY;i++)
      if(joydata[i].fd >= 0)
         close(joydata[i].fd);
}

void osd_trak_read(int player,int *deltax,int *deltay)
{
   if (player == 0 && use_mouse)
   {
      *deltax = mouse_delta_x;
      *deltay = mouse_delta_y;
   }
   else
   {
      *deltax = 0;
      *deltay = 0;
   }
}

void osd_poll_joysticks (void)
{
   if (use_mouse)
   {
      sysdep_mouse_poll ();
      mouse_joy_x += mouse_delta_x;
      mouse_joy_y += mouse_delta_y;

      if (mouse_joy_x > 128)
         mouse_joy_x = 128;
      else if (mouse_joy_x < -128)
         mouse_joy_x = -128;

      if (mouse_joy_y > 128)
         mouse_joy_y = 128;
      else if (mouse_joy_y < -128)
         mouse_joy_y = -128;
   }
   if (joytype)
      (*joy_poll_func) ();
}

int osd_joy_pressed (int joycode)
{
   int i, joy_no = 0;

   /* if a joystick isn't attached the joydata struct is set to 0,
      we can't just abort since this code is also used to check mouse
      clicks */

   if (joycode > OSD_MAX_JOY)
      return FALSE;
   while (joycode > OSD_JOY_FIRE)  /* OSD_JOY_FIRE is the last one in the enum */
   {
      joycode -= OSD_JOY_FIRE;
      joy_no++;
   }

   switch (joycode)
   {
      case OSD_JOY_LEFT:
         return joydata[joy_no].left;
      case OSD_JOY_RIGHT:
         return joydata[joy_no].right;
      case OSD_JOY_UP:
         return joydata[joy_no].up;
      case OSD_JOY_DOWN:
         return joydata[joy_no].down;
      case OSD_JOY_FIRE1:
         if (joy_no == 0)
            return (joydata[0].buttons[0] || mouse_b1);
         return joydata[joy_no].buttons[0];
      case OSD_JOY_FIRE2:
         if (joy_no == 0)
            return (joydata[0].buttons[1] || mouse_b2);
         return joydata[joy_no].buttons[1];
      case OSD_JOY_FIRE3:
         if (joy_no == 0)
            return (joydata[0].buttons[2] || mouse_b3);
         return joydata[joy_no].buttons[2];
      case OSD_JOY_FIRE4:
      case OSD_JOY_FIRE5:
      case OSD_JOY_FIRE6:
      case OSD_JOY_FIRE7:
      case OSD_JOY_FIRE8:
      case OSD_JOY_FIRE9:
      case OSD_JOY_FIRE10:
         return joydata[joy_no].buttons[joycode - OSD_JOY_FIRE1];
      case OSD_JOY_FIRE:
         for (i = 0; i < 10; i++)
            if (joydata[joy_no].buttons[i])
               return TRUE;
         return (mouse_b1 || mouse_b2 || mouse_b3);
      default:
         return FALSE;
   }
}

/* return the name of a joystick button */
const char *osd_joy_name (int joycode)
{
   static char *joynames[] =
   {"None",
    "Left", "Right", "Up", "Down", "Button A",
    "Button B", "Button C", "Button D", "Button E", "Button F",
    "Button G", "Button H", "Button I", "Button J", "Any Button",
    "J2 Left", "J2 Right", "J2 Up", "J2 Down", "J2 Button A",
   "J2 Button B", "J2 Button C", "J2 Button D", "J2 Button E", "J2 Button F",
    "J2 Button G", "J2 Button H", "J2 Button I", "J2 Button J", "J2 Any Button",
    "J3 Left", "J3 Right", "J3 Up", "J3 Down", "J3 Button A",
   "J3 Button B", "J3 Button C", "J3 Button D", "J3 Button E", "J3 Button F",
    "J3 Button G", "J3 Button H", "J3 Button I", "J3 Button J", "J3 Any Button",
    "J4 Left", "J4 Right", "J4 Up", "J4 Down", "J4 Button A",
   "J4 Button B", "J4 Button C", "J4 Button D", "J4 Button E", "J4 Button F",
    "J4 Button G", "J4 Button H", "J4 Button I", "J4 Button J", "J4 Any Button"
   };

   if (joycode <= OSD_MAX_JOY)
      return (char *) joynames[joycode];
   else
      return "Unknown";
}

/*
 * given a new x an y joystick axis value convert it to a move definition
 */

void joy_evaluate_moves (void)
{
   int i;
   int x_treshold, y_treshold;

   for (i = 0; i < MAX_JOY; i++)
   {
      if (!joydata[i].present)
         continue;
      
      joydata[i].up = joydata[i].down = 0;
      joydata[i].left = joydata[i].right = 0;

      /* auto calibrate */
      if (joydata[i].x > joydata[i].max_x)
         joydata[i].max_x = joydata[i].x;
      else if (joydata[i].x < joydata[i].min_x)
         joydata[i].min_x = joydata[i].x;
      if (joydata[i].y > joydata[i].max_y)
         joydata[i].max_y = joydata[i].y;
      else if (joydata[i].y < joydata[i].min_y)
         joydata[i].min_y = joydata[i].y;

      x_treshold = (joydata[i].max_x - joydata[i].center_x) >> 1;
      y_treshold = (joydata[i].max_y - joydata[i].center_y) >> 1;

      if (joydata[i].x > (joydata[i].center_x + x_treshold))
         joydata[i].right = 1;
      else if (joydata[i].x < (joydata[i].center_x - x_treshold))
         joydata[i].left = 1;
      if (joydata[i].y > (joydata[i].center_y + y_treshold))
         joydata[i].down = 1;
      else if (joydata[i].y < (joydata[i].center_y - y_treshold))
         joydata[i].up = 1;
   }
}

/* 
 * return a value in the range -128 .. 128 (yes, 128, not 127)
 */
void osd_analogjoy_read(int player,int *analog_x, int *analog_y)
{
   if (player < MAX_JOY)
   {
      if (joydata[player].present && analogstick)
      {
         if (joytype == JOY_I386NEW)
         {
            *analog_x = joydata[player].x / 256;
            *analog_y = joydata[player].y / 256;
         }
         else
         {
            if (joydata[player].x > joydata[player].center_x)
               *analog_x = ((128 * (joydata[player].x - joydata[player].center_x)) /
                            (joydata[player].max_x - joydata[player].center_x));
            else
               *analog_x = ((128 * (joydata[player].x - joydata[player].center_x)) /
                            (joydata[player].center_x - joydata[player].min_x));
   
            if (joydata[player].y > joydata[player].center_y)
               *analog_y = ((128 * (joydata[player].y - joydata[player].center_y)) /
                            (joydata[player].max_y - joydata[player].center_y));
            else
               *analog_y = ((128 * (joydata[player].y - joydata[player].center_y)) /
                            (joydata[player].center_y - joydata[player].min_y));
         }
         return;
      }
      if (player == 0 && use_mouse && analogmouse)
      {
         *analog_x = mouse_joy_x;
         *analog_y = mouse_joy_y;
         return;
      }
   }
   *analog_x = *analog_y = 0;
}

int osd_joystick_needs_calibration (void)
{
   /* xmame uses the kernels joystick drivers calibration, or autocalibration
      and thus never needs this */
   return 0;
}

void osd_joystick_start_calibration (void)
{
}

char *osd_joystick_calibrate_next (void)
{
   return NULL;
}

void osd_joystick_calibrate (void)
{
}

void osd_joystick_end_calibration (void)
{
}
