#ifdef I386_JOYSTICK

#include "xmame.h"
#include "devices.h"
#include <sys/ioctl.h>

/* specific joystick for PC clones */
#ifdef netbsd_i386

#include <machine/joystick.h>
typedef struct joystick joy_struct;

#elif defined linux_i386 || defined linux_alpha

#include <linux/joystick.h>
typedef struct JS_DATA_TYPE joy_struct;

#ifdef JS_VERSION
#define I386NEW_JOYSTICK 1
#endif

#else
#error "i386 style joystick only supported under linux & netbsd-i386, "
"patches to support other arch's should be easy and are welcome ;)"
#endif

void joy_i386_poll(void);
void joy_i386new_poll(void);
static joy_struct joy_data;

void joy_i386_init (void)
{
   int i, found = 0;
   char devname[20];
#ifdef I386NEW_JOYSTICK
   int version;
#endif

   fprintf (stderr_file, "I386 joystick interface initialization...\n");
   for (i = 0; i < MAX_JOY; i++)
   {
      sprintf (devname, "/dev/js%d", i);
      if ((joydata[i].fd = open (devname, O_RDONLY)) >= 0)
      {
         if (read(joydata[i].fd, &joy_data, sizeof(joy_struct)) != sizeof(joy_struct))
         {
            close(joydata[i].fd);
            joydata[i].fd = -1;
            continue;
         }
         switch(joytype)
         {
            case JOY_I386NEW:
#ifdef I386NEW_JOYSTICK
               /* new joystick driver 1.x.x API 
                  check the running version of driver, if 1.x.x is
                  not detected fall back to 0.8 API */

               if (ioctl (joydata[i].fd, JSIOCGVERSION, &version)==0)
               {
                  char axes;
                  char buttons;
                  char name[60];
                  ioctl (joydata[i].fd, JSIOCGAXES, &axes);
                  ioctl (joydata[i].fd, JSIOCGBUTTONS, &buttons);
                  ioctl (joydata[i].fd, JSIOCGNAME (sizeof (name)), name);
                  fprintf (stderr_file, "Joystick: %s is %s\n", devname, name);
                  fprintf (stderr_file, "Joystick: Built in driver version: %d.%d.%d\n", JS_VERSION >> 16, (JS_VERSION >> 8) & 0xff, JS_VERSION & 0xff);
                  fprintf (stderr_file, "Joystick: Kernel driver version  : %d.%d.%d\n", version >> 16, (version >> 8) & 0xff, version & 0xff);
                  joydata[i].center_y = joydata[i].center_x = 0;
                  joy_poll_func = joy_i386new_poll;
                  break;
               }
               /* else we're running on a kernel with 0.8 driver */
               fprintf (stderr_file, "Joystick: %s unknown type\n", devname);
               fprintf (stderr_file, "Joystick: Built in driver version: %d.%d.%d\n", JS_VERSION >> 16, (JS_VERSION >> 8) & 0xff, JS_VERSION & 0xff);
               fprintf (stderr_file, "Joystick: Kernel driver version  : 0.8 ??\n");
               fprintf (stderr_file, "Joystick: Please update your Joystick driver !\n");
               fprintf (stderr_file, "Joystick: Using old interface method\n");
#else
               fprintf (stderr_file, "New joystick driver (1.x.x) support not compiled in.\n");
               fprintf (stderr_file, "Falling back to 0.8 joystick driver api\n");
#endif            
               joytype = JOY_I386;
            case JOY_I386:
               /* mask x e y pos to skip "flickering" */
               if (swapjoyaxis)
               {
                  joydata[i].center_y = joy_data.x;
                  joydata[i].center_x = joy_data.y;
               }
               else
               {
                  joydata[i].center_x = joy_data.x;
                  joydata[i].center_y = joy_data.y;
               }
               joy_poll_func = joy_i386_poll;
               break;
         }
         found++;
         joydata[i].present=1;
         fcntl (joydata[i].fd, F_SETFL, O_NONBLOCK);
      }
   }
   fprintf (stderr_file, "Joystick: Found %d joysticks\n", found);
   if (found == 0)
   {
      fprintf (stderr_file, "Joystick: Be sure joystick is configured into kernel\n");
      joytype = JOY_NONE;
   }
}

#ifdef I386NEW_JOYSTICK
/* 
 * Routine to manage PC clones joystick via new Linux driver 1.2.xxx
 */
void joy_i386new_poll (void)
{
   struct js_event js;
   int i;

   for (i = 0; i < MAX_JOY; i++)
   {
      if (!joydata[i].present)
         continue;
      if ((read (joydata[i].fd, &js, sizeof (struct js_event))) != sizeof (struct js_event))
           continue;

      switch (js.type & ~JS_EVENT_INIT)
      {
         case JS_EVENT_BUTTON:
            joydata[i].buttons[js.number] = js.value;
#ifdef JDEBUG
            fprintf (stderr, "\rButton=%d,value=%d", js.number, js.value);
#endif
            break;

         case JS_EVENT_AXIS:
            if (js.number == 0)
               joydata[i].x = js.value;
            if (js.number == 1)
               joydata[i].y = js.value;
#ifdef JDEBUG
            fprintf (stderr, "\raxes=%d,value=%d                 ", js.number, js.value);
#endif
            break;
      }
      /* now parse movements */
      if (swapjoyaxis)
      {
         int a = joydata[i].y;

         joydata[i].y = joydata[i].x;
         joydata[i].x = a;
      }
      /* evaluate joystick movements */
      joydata[i].up = joydata[i].down = 0;
      joydata[i].left = joydata[i].right = 0;
      if (joydata[i].x > 16384)
         joydata[i].right = 1;
      else if (joydata[i].x < -16384)
         joydata[i].left = 1;
      if (joydata[i].y > 16384)
         joydata[i].down = 1;
      else if (joydata[i].y < -16384)
         joydata[i].up = 1;
   }
}
#endif

/* 
 * Routine to manage PC clones joystick via standard driver 
 */
void joy_i386_poll (void)
{
   int i, j, res;

   for (i = 0; i < MAX_JOY; i++)
   {
      if (!joydata[i].present)
         continue;
      if (read (joydata[i].fd, &joy_data, sizeof (joy_struct)) != sizeof (joy_struct))
         continue;
      
      /* get value of buttons */
#ifdef netbsd_i386
      joydata[i].buttons[0] = joy_data.b1;
      joydata[i].buttons[1] = joy_data.b2;
#else
      for (j = 0; j < 10; j++)
         joydata[i].buttons[j] = joy_data.buttons & (0x01 << j);
#endif
      /* now parse movements */
      if (swapjoyaxis)
      {
         joydata[i].y = joy_data.x;
         joydata[i].x = joy_data.y;
      }
      else
      {
         joydata[i].x = joy_data.x;
         joydata[i].y = joy_data.y;
      }
   }

   /* evaluate joystick movements */
   joy_evaluate_moves ();
}

#endif
