/*
 ************************** FM TOWNS PAD specifics routines *************
 */
#ifdef LIN_FM_TOWNS

#include <fcntl.h>
#include "xmame.h"
#include "devices.h"
#include "pad.h"

/*
 * Towns Pad control module for Xmame
 *
 * Author : Osamu KURATI
 * Version : 29 apr 1997 0.000
 */

/*
 * Joy Stick Code
 * Up : 1
 * Down       : 2
 * Left       : 4
 * Right      : 8
 * A  : 10
 * B  : 20
 *
 *
 * PAD bit number
 * up         : 0
 * down               : 1
 * left               : 2
 * right      : 3
 * A          : 4
 * B          : 5
 * RUN                : 6
 * SELECT     : 7
 */

static unsigned long lPadLastButton = 0;
void joy_pad_poll(void);

void joy_pad_init(void)
{
  int i;
  
  analogstick   = FALSE;
  joy_poll_func = joy_pad_poll;
 
  if ((joydata[0].fd = open(towns_pad_dev, O_NONBLOCK | O_RDONLY)) < 0){
    fprintf(stderr_file,"Unable to open FM-TOWNS game pad. Joystick disabled\n");
    joytype = JOY_NONE;
  }
  else
    joydata[0].present = 1;
  
  lPadLastButton = 0;
}

static int Pad()
{
  struct pad_event ev;
  if (read(joydata[0].fd, &ev, sizeof ev) == sizeof ev){
    lPadLastButton = ev.buttons;
  }
  return((int) lPadLastButton & 0xff);
}

/*
 * Linux FM-TOWNS game pad driver based joystick emulation routine
 */
void joy_pad_poll(void)
{
      int i;
      int res = Pad();

      /* get value of buttons */
      for (i=0; i<4; i++)
      {
         joydata[0].buttons[i] = (res>>4) & (0x01<<i);
      }

      joydata[0].up    = res & 0x01;
      joydata[0].down  = res & 0x02;
      joydata[0].left  = res & 0x04;
      joydata[0].right = res & 0x08;
}

/* TOWNS_PAD */
#endif
