/*
 * NetBSD/i386 audio code by entropy@zippy.bernstein.com
 * Audio code is based on the solaris driver, by jantonio@dit.upm.es
 *
 * Should be replaced by generic solaris (/dev/audio and /dev/audioctl) support
 */

#include "xmame.h"
#include "sound.h"
#include "devices.h"
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/audioio.h>
static int audio_fd;
static audio_info_t a_info;
static audio_device_t a_dev;

int sysdep_audio_init(void)
{
  if (play_sound)
  {
    fprintf(stderr_file, "NetBSD/i386 sound device initialization...\n");
    /* try to open audio device */
    if ((audio_fd = open("/dev/audio", O_WRONLY /* | O_NDELAY*/)) < 0)
    {
      perror("Cannot open audio device");
      play_sound = FALSE;
    }
    else
    {
      /* empty buffers before change config */
      ioctl(audio_fd, AUDIO_FLUSH, 0);        /* flush everything */
      
      /* identify audio device. */
      if(ioctl(audio_fd, AUDIO_GETDEV, &a_dev) < 0)
      {
	perror("Cannot get sound device type");
	close(audio_fd);
	play_sound = FALSE;
      }
      else
      {
	fprintf(stderr_file, "Sound device is a %s %s version %s\n",
	       a_dev.config, a_dev.name, a_dev.version);
	    
	/* initialize audio parameters. */ 
	AUDIO_INITINFO(&a_info);
	    
	a_info.play.sample_rate = (uint) options.samplerate = AUDIO_SAMPLE_FREQ;
	a_info.play.encoding = (uint) AUDIO_ENCODING_ULINEAR;
	a_info.play.precision = (uint) AUDIO_SAMPLE_BITS;
	a_info.play.channels = (uint) 1;
	a_info.blocksize = AUDIO_SAMPLE_FREQ / AUDIO_TIMER_FREQ;
/*	a_info.play.buffer_size = AUDIO_BUFF_SIZE; */ /* Not settable */
	a_info.mode = AUMODE_PLAY | AUMODE_PLAY_ALL;

	if (ioctl(audio_fd, AUDIO_SETINFO, &a_info) < 0)
	{
	  perror("cannot set audio device parameters");
	  close(audio_fd);
	  play_sound = FALSE;
	}
      }
    }
  }
  return OSD_OK;
}

void sysdep_audio_close(void)
{
  if (play_sound) close(audio_fd);
}

int sysdep_audio_play(unsigned char *buf, int bufsize)
{
  return write(audio_fd, buf, bufsize);
}

long sysdep_audio_get_freespace()
{
#ifndef USE_TIMER
       audio_info_t info;
       int i;
       long l;
        AUDIO_INITINFO(&info);
        i = ioctl(audio_fd,AUDIO_GETINFO,&info);
       if (i<0) { perror("AUDIO_GETINFO"); return -1; }
       l = AUDIO_BUFF_SIZE - info.play.seek;
       return (l < 0) ? 0 : l;
#else 
       return 0;
#endif   
}
