/***************************************************************************

  Xmame 3Dfx console-mode driver

  Written based on Phillip Ezolt's svgalib driver by Mike Oliphant -

    oliphant@ling.ed.ac.uk

    http://www.ling.ed.ac.uk/~oliphant/glmame

***************************************************************************/
#define __SVGAFX_C

#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
#include <vgamouse.h>
#include <signal.h>

#include "xmame.h"
#include "devices.h"

extern void InitVScreen(void);
extern void CloseVScreen(void);

int fx=1;
int winwidth=640;
int winheight=480;

static struct sigaction sig2handler;
static struct sigaction oldsig2handler;
#if (__GLIBC__ > 1)
static sigset_t sig2sigset;
#endif

int sysdep_init(void)
{
   vga_init();
   return OSD_OK;
}

void Sig2HandlerFunction(int n)
{
   keyboard_clearstate();
   (*(oldsig2handler.sa_handler))(n);
   sigaction(SIGUSR2, &sig2handler, NULL);
}


/* This name doesn't really cover this function, since it also sets up mouse
   and keyboard. This is done over here, since on most display targets the
   mouse and keyboard can't be setup before the display has. */
int sysdep_create_display(void)
{
  char gammastr[14];

  printf("Using FXmame v0.5 driver for xmame, written by Mike Oliphant\n");
  
  putenv("FX_GLIDE_NO_SPLASH=");

  sprintf(gammastr,"SST_GAMMA=%2.1f",fxgamma);

  putenv(gammastr);

#if (__GLIBC__ > 1)
   memset(&sig2sigset,0,sizeof(sigset_t));
   sig2handler.sa_mask=sig2sigset;
#else	
   sig2handler.sa_mask=0;
#endif
   sig2handler.sa_handler=Sig2HandlerFunction;
   sig2handler.sa_flags=0;
	
   sigaction(SIGUSR2, &sig2handler, &oldsig2handler);

   /* init the keyboard */
   keyboard_init();
   local_key=keyboard_getstate();

   /* 
      not sure if this is the best site but mouse init routine must be 
      called after video initialization...
   */   
   if(use_mouse)
   {
	int fd;
	vga_setmousesupport(TRUE);
	fd=mouse_init_return_fd("/dev/mouse",vga_getmousetype(),MOUSE_DEFAULTSAMPLERATE);
	if (fd<0)
	{
		perror("mouse_init");
		fprintf(stderr_file,"SVGALib: failed to open mouse device %d\n",fd);
		use_mouse=0;
	}
	else
	{
		/* fix ranges and initial position of mouse */
		mouse_setxrange(-500,500);
		mouse_setyrange(-500,500);
		mouse_setposition(0,0);
	}
   }
   
#ifdef USE_TIMER
    /* svgalib catches the timer alarm. so we have to arm it after all
       other svgalib initialising. */
    if (play_sound)
    {
       if(start_timer()==OSD_NOT_OK)
       {
          osd_close_display();
          return OSD_NOT_OK;
       }
    }
#endif
   
  InitVScreen();

  return OSD_OK;
}


/* shut up the display */
void osd_close_display(void)
{
   sigaction(SIGUSR2, &oldsig2handler, NULL);
   keyboard_close();
   osd_free_bitmap(bitmap);
   CloseVScreen();  /* Shut down glide stuff */
   if (use_mouse) mouse_close();
}

int sysdep_mapkey(int from, int to) 
{
	return OSD_OK;
}

void sysdep_mouse_poll (void)
{
	int mouse_buttons;
	
	mouse_update();
	
	mouse_delta_x = mouse_getx();
	mouse_delta_y = mouse_gety();
	
	mouse_buttons = mouse_getbutton();
	
	mouse_b1 = (MOUSE_LEFTBUTTON & mouse_buttons)?   1 : 0;
	mouse_b2 = (MOUSE_RIGHTBUTTON & mouse_buttons)?  1 : 0;
	mouse_b3 = (MOUSE_MIDDLEBUTTON & mouse_buttons)? 1 : 0;
	
	mouse_setposition(0,0);
}
