/***************************************************************************

 Linux SVGALib adaptation by Phillip Ezolt pe28+@andrew.cmu.edu
  
***************************************************************************/
#define __SVGALIB_C

#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
#include <vgamouse.h>
#include <signal.h>
#include <linux/kd.h>
#include <sys/ioctl.h>
#include "xmame.h"
#include "devices.h"

/* for the tweaked modes */
#ifndef linux_powerpc
#include "twkuser.c"
#include "twkmodes.h"
#else
typedef void Register;
#endif

static int startx, starty;
static int scaled_visual_width, scaled_visual_height;
static int video_mode = -1;
static unsigned char *video_mem = NULL;
static int text_mode = TRUE;
static void (*svgalib_update_display_func)(void) = NULL;
static char *doublebuffer_buffer = NULL;
static int console_fd = -1;
static struct sigaction sig2handler;
static struct sigaction oldsig2handler;
#if (__GLIBC__ > 1)
static sigset_t sig2sigset;
#endif
static void svgalib_update_linear(void);
static void svgalib_update_planar(void);
static void svgalib_update_gl(void);
static void svgalib_update_gl_scaled(void);

typedef enum { MODE_NORMAL, MODE_PLANAR} mode_type;

struct video_mode_struct
{
   int width;
   int height;
   int svga_mode;
   mode_type mode;
   Register *registers;
   Register *registers_scanline;
   int use_mode;
};

/* all available videomodes listed with the smallest ones in height first */
static struct video_mode_struct video_modes[] = {
#ifndef linux_powerpc
{  320,  200,   G320x200x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{  320,  204,   G320x200x256, MODE_NORMAL, scr320x204, NULL,                TRUE },
{  288,  224,   G320x200x256, MODE_NORMAL, scr288x224, scr288x224scanlines, TRUE },
{  320,  240,   G320x240x256, MODE_PLANAR, NULL,       NULL,                TRUE },
{  256,  256,   G320x200x256, MODE_NORMAL, scr256x256, scr256x256scanlines, TRUE },
{  240,  272,   G320x200x256, MODE_NORMAL, scr240x272, NULL,                TRUE },
{  224,  288,   G320x200x256, MODE_NORMAL, scr224x288, scr224x288scanlines, TRUE },
{  200,  320,   G320x200x256, MODE_NORMAL, scr200x320, NULL,                TRUE },
{  320,  400,   G320x400x256, MODE_PLANAR, NULL,       NULL,                TRUE },
/* seems broken anyone care to fix it ? */
{  360,  480,   G360x480x256, MODE_PLANAR, NULL,       NULL,                FALSE },
#endif
{  640,  480,   G640x480x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{  800,  600,   G800x600x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{ 1024,  768,  G1024x768x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{ 1152,  864,  G1152x864x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{ 1280, 1024, G1280x1024x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{ 1600, 1200, G1600x1200x256, MODE_NORMAL, NULL,       NULL,                TRUE },
{   0,     0,              0,        0,    NULL,       NULL,                TRUE }
};

int sysdep_init(void)
{
   vga_init();
   return OSD_OK;
}

void sysdep_disable_mode(char *mode_string)
{
   int i, width=0, height=0;
   sscanf(mode_string, "%dx%d", &width, &height);
   for (i=0; video_modes[i].width; i++)
   {
      if (video_modes[i].width == width && video_modes[i].height == height)
      {
         video_modes[i].use_mode = FALSE;
         break;
      }
   }
   if (video_modes[i].width == 0)
   {
      /* stderr_file doesn't have a valid value yet when we're called ! */
      fprintf(stderr, "Warning invalid mode string, or no such mode: %s. Mode not disabled\n",
         mode_string);
   }
}

#ifndef linux_powerpc
static void set_tweaked_mode(void)
{
	if (!text_mode && video_modes[video_mode].registers)
	{
	   if (use_scanlines && video_modes[video_mode].registers_scanline)
	      outRegArray(video_modes[video_mode].registers_scanline, 25);
	   else
	      outRegArray(video_modes[video_mode].registers, 25);
	}
}
#endif

int sysdep_set_video_mode (void)
{
   int i;
   
   if (!text_mode) return OSD_NOT_OK;
   if (!vga_hasmode(video_modes[video_mode].svga_mode))
   {
      fprintf(stderr_file,"Svgalib mode: %d (%d x %d) not available\n",
         video_modes[video_mode].svga_mode, video_modes[video_mode].width,
         video_modes[video_mode].height);
      return OSD_NOT_OK;
   }    
   vga_setmode(video_modes[video_mode].svga_mode);
   gl_setcontextvga(video_modes[video_mode].svga_mode);
   text_mode = FALSE;

   if (video_modes[video_mode].mode == MODE_NORMAL)
   {
#ifndef linux_powerpc
      /* do we have a linear framebuffer ? */
      i = video_modes[video_mode].width * video_modes[video_mode].height;
      if (i <= 65536 || (use_linear && vga_setlinearaddressing() >=  i))
      {
         video_mem  = vga_getgraphmem();
         video_mem += startx;
         video_mem += starty * video_modes[video_mode].width;
         set_tweaked_mode();
         if (widthscale != 1 || heightscale != 1)
         {
            doublebuffer_buffer = malloc(scaled_visual_width);
            if (!doublebuffer_buffer)
            {
               fprintf(stderr_file, "Svgalib: Error: couldn't allocate doublebuffer buffer\n");
               return OSD_NOT_OK;
            }
         }
         svgalib_update_display_func=svgalib_update_linear;
         fprintf(stderr_file, "Svgalib: Using a linear framebuffer to speed up\n");
      }
      else
#endif
      if(widthscale==1 && heightscale==1)
         svgalib_update_display_func=svgalib_update_gl;
      else
      {
         if (!video_mem) /* with the debugger we can be called more then once ! */
            video_mem = malloc(scaled_visual_width*scaled_visual_height);
         if (!video_mem)
         {
            fprintf(stderr_file, "Svgalib: couldn't allocate screen buffer\n");
            return OSD_NOT_OK;
         }
         svgalib_update_display_func=svgalib_update_gl_scaled;
      }
   }
   else
      svgalib_update_display_func=svgalib_update_planar;

   sysdep_mark_palette_dirty();
   osd_mark_dirty (0,0,bitmap->width-1,bitmap->height-1,1);
   
   return OSD_OK;
}

void sysdep_set_text_mode (void)
{
   if (text_mode) return;
   vga_setmode(TEXT);
   text_mode=TRUE;
}

void Sig2HandlerFunction(int n)
{
   keyboard_clearstate();
   (*(oldsig2handler.sa_handler))(n);
#ifndef linux_powerpc   
   set_tweaked_mode();
#endif
   sigaction(SIGUSR2, &sig2handler, NULL);
}

/* This name doesn't really cover this function, since it also sets up mouse
   and keyboard. This is done over here, since on most display targets the
   mouse and keyboard can't be setup before the display has. */
int sysdep_create_display(void)
{
   int i;
   
   scaled_visual_width  = visual_width  * widthscale;
   scaled_visual_height = visual_height * heightscale;
   
   for (i=0; video_modes[i].width; i++)
   {
      if(video_modes[i].registers && !use_tweak)
         video_modes[i].use_mode = FALSE;
      if(video_modes[i].mode == MODE_PLANAR &&
         (!use_planar || widthscale != 1 || heightscale != 1))
         video_modes[i].use_mode = FALSE;
      if(video_modes[i].use_mode &&
         scaled_visual_width  <= video_modes[i].width &&
         scaled_visual_height <= video_modes[i].height)
      {
         video_mode = i;
         break;
      }
   }
   
   if (video_modes[i].width == 0)
   {
      fprintf(stderr_file, "Svgalib: Couldn't find a suitable mode for a resolution of %d x %d\n",
         scaled_visual_width, scaled_visual_height);
      return OSD_NOT_OK;
   }
   
   startx = ((video_modes[video_mode].width  - scaled_visual_width ) / 2) & ~7;
   starty =  (video_modes[video_mode].height - scaled_visual_height) / 2;
   
   if (sysdep_set_video_mode()!=OSD_OK) return OSD_NOT_OK;
   
   fprintf(stderr_file, "Using a mode with a resolution of %d x %d, starting at %d x %d\n",
      video_modes[video_mode].width, video_modes[video_mode].height,
      startx, starty);
   
#if (__GLIBC__ > 1)
   memset(&sig2sigset,0,sizeof(sigset_t));
   sig2handler.sa_mask=sig2sigset;
#else	
   sig2handler.sa_mask=0;
#endif
   sig2handler.sa_handler=Sig2HandlerFunction;
   sig2handler.sa_flags=0;
	
   sigaction(SIGUSR2, &sig2handler, &oldsig2handler);
   
   /* set the palette to black */
   for (i=0; i < 256; i++) gl_setpalettecolor(i, 0, 0, 0);

#ifdef USE_TIMER
    /* svgalib catches the timer alarm. so we have to arm it after all
       other svgalib initialising. */
    if (play_sound)
    {
       if(start_timer()==OSD_NOT_OK)
       {
          osd_close_display();
          return OSD_NOT_OK;
       }
    }
#endif

   /* init the keyboard */
   if ((console_fd = keyboard_init_return_fd()) < 0)
   {
      fprintf(stderr_file, "Svgalib: Error: Couldn't open keyboard\n");
      return OSD_NOT_OK;
   }
   ioctl(console_fd, KDSETLED, 0);
   local_key=keyboard_getstate();

   /* init the mouse */
   if(use_mouse)
   {
	int fd;
	vga_setmousesupport(TRUE);
	fd=mouse_init_return_fd("/dev/mouse",vga_getmousetype(),MOUSE_DEFAULTSAMPLERATE);
	if (fd<0)
	{
		perror("mouse_init");
		fprintf(stderr_file,"SVGALib: failed to open mouse device %d\n",fd);
		use_mouse=0;
	}
	else
	{
		/* fix ranges and initial position of mouse */
		mouse_setxrange(-500,500);
		mouse_setyrange(-500,500);
		mouse_setposition(0,0);
	}
   }
   
   return OSD_OK;
}

/* shut up the display */
void osd_close_display(void)
{
   sigaction(SIGUSR2, &oldsig2handler, NULL);
   if (console_fd >= 0)
   {
      ioctl(console_fd, KDSETLED, 8);
      keyboard_close();
   }
   sysdep_set_text_mode();
   if (svgalib_update_display_func==svgalib_update_gl_scaled)
      free(video_mem);
   if (doublebuffer_buffer)
      free(doublebuffer_buffer);
   osd_dirty_close();
   /* free the bitmap after cleaning the dirty stuff as it uses the bitmap */
   osd_free_bitmap(bitmap);
}

/*
 * Alloc global symbol totalcolors colors, not needed under
 * svgalib, under svgalib we always have access to all available colors
 */
void sysdep_alloc_palette(void)
{
}

void sysdep_modify_pen(int pen,unsigned char red, unsigned char green, unsigned char blue)
{
   gl_setpalettecolor(pen,(red>>2),(green>>2),(blue>>2));
}

/* Update the display. */
void sysdep_update_display(void)
{
   (*svgalib_update_display_func)();
}

static void svgalib_update_linear(void)
{
#define DEST video_mem
#define DEST_PIXEL unsigned char
#define DEST_WIDTH video_modes[video_mode].width
#define DOUBLEBUFFER
#include "blit.h"
#undef DEST
#undef DEST_PIXEL
#undef DEST_WIDTH
#undef DOUBLEBUFFER
}

static void svgalib_update_planar(void)
{
  /* use page flipping otherwise the screen tears in planar modes,
     unfortunatly this also means we can't use dirty in planar modes */
  static int page=0;
  if (page) page=0;
   else page=131072;
  vga_copytoplanar256(bitmap->line[visual.min_y] + visual.min_x,
     bitmap->line[1] - bitmap->line[0],
     (starty*video_modes[video_mode].width + startx + page)/4, 
      video_modes[video_mode].width/4, visual_width, visual_height);
  vga_setdisplaystart(page);
}

static void svgalib_update_gl(void)
{
#define PUT_IMAGE(X, Y, WIDTH, HEIGHT) \
      gl_putboxpart( \
         startx + X, starty + Y, \
         WIDTH, HEIGHT, \
         bitmap->line[1] - bitmap->line[0], bitmap->height, \
         bitmap->line[0], X + visual.min_x, Y + visual.min_y);
         /* Note: we calculate the real bitmap->width, as used in
            osd_create_bitmap, this fixes the scewing bug in tempest
            & others */
#include "blit.h"
#undef PUT_IMAGE
}

static void svgalib_update_gl_scaled(void)
{
#define DEST video_mem
#define DEST_PIXEL unsigned char
#define DEST_WIDTH scaled_visual_width
#define PUT_IMAGE(X, Y, WIDTH, HEIGHT) \
      gl_putboxpart( \
         startx + X, starty + Y, \
         WIDTH, HEIGHT, \
         scaled_visual_width, scaled_visual_height, \
         video_mem, X, Y );
#include "blit.h"
#undef DEST
#undef DEST_PIXEL
#undef DEST_WIDTH
#undef PUT_IMAGE
}

int sysdep_mapkey(int from, int to) 
{
   return OSD_OK;
}

void sysdep_mouse_poll (void)
{
	int mouse_buttons;
	
	mouse_update();
	
	mouse_delta_x = mouse_getx();
	mouse_delta_y = mouse_gety();
	
	mouse_buttons = mouse_getbutton();
	
	mouse_b1 = (MOUSE_LEFTBUTTON & mouse_buttons)?   1 : 0;
	mouse_b2 = (MOUSE_RIGHTBUTTON & mouse_buttons)?  1 : 0;
	mouse_b3 = (MOUSE_MIDDLEBUTTON & mouse_buttons)? 1 : 0;
	
	mouse_setposition(0,0);
}

static const int led_flags[3] = {
  LED_NUM,
  LED_CAP,
  LED_SCR
};

void osd_led_w(int led,int on)
{
	static int leds=0;
	if (led>=3) return;
	if (on)
		leds |=  led_flags[led];
	else
		leds &= ~led_flags[led];
	if (console_fd >= 0)
	{
	   ioctl(console_fd, KDSETLED, leds);
	}
}
