/*
 * X-Mame video specifics code
 *
 */
#ifdef x11
#define __X11_C_

/*
 * Include files.
 */

#include "xmame.h"
#include "x11.h"
#include "input.h"

static int  (*x_create_display_func)(void) = NULL;
static void (*x_close_display_func)(void) = NULL;
static void (*x_modify_pen_func)(int pen, unsigned char red,unsigned char green,unsigned char blue) = NULL;
static void (*x_update_display_func)(void) = NULL;
static void (*x_alloc_palette_func)(void) = NULL;

int sysdep_init (void)
{
#ifdef USE_DGA
   return xf86_dga_init();
#else
   xf86_dga_available = 0;
   return OSD_OK;
#endif
}

/* This name doesn't really cover this function, since it also sets up mouse
   and keyboard. This is done over here, since on most display targets the
   mouse and keyboard can't be setup before the display has. */
int sysdep_create_display (void)
{
#ifndef USE_DGA /* xf86_dga_init already opens the display */
   if(!(display = XOpenDisplay (NULL)))
   {
      fprintf (stderr_file, "Could not open display\n");
      return OSD_NOT_OK;
   }
#endif

   /* first setup the keyboard that's the same for all X video modes */
   local_key=xkey;
   memset((void *)&xkey[0], FALSE, 128*sizeof(unsigned char) );

   if ((x11_video_mode == X11_DGA) && (xf86_dga_available == 0))
   {
      fprintf (stderr, "DGA support not available, falling back to normal window code\n");
      x11_video_mode = X11_WINDOW;
   }

   switch (x11_video_mode)
   {
      case X11_WINDOW:
         x_create_display_func = x11_window_create_display;
         x_close_display_func  = x11_window_close_display;
         x_modify_pen_func     = x11_window_modify_pen;
         x_update_display_func = x11_window_update_display;
         x_alloc_palette_func  = x11_window_alloc_palette;
         break;
#ifdef USE_DGA
      case X11_DGA:
         x_create_display_func = xf86_dga_create_display;
         x_close_display_func  = xf86_dga_close_display;
         x_modify_pen_func     = xf86_dga_modify_pen;
         x_update_display_func = xf86_dga_update_display;
         x_alloc_palette_func  = xf86_dga_alloc_palette;
         break;
#endif
      default:
         fprintf (stderr_file, "Error: Unknown x11-videomode: %d\n", x11_video_mode);
         XCloseDisplay (display);
         display = 0;
         return OSD_NOT_OK;
   }

   return (*x_create_display_func) ();
}

void osd_close_display (void)
{
   if(x_close_display_func)
      (*x_close_display_func) ();
   if(display)
      XCloseDisplay (display);
   osd_dirty_close ();
   /* free the bitmap after cleaning the dirty stuff as it uses the bitmap */
   osd_free_bitmap (bitmap);
}

void sysdep_alloc_palette (void)
{
   (*x_alloc_palette_func) ();
}

void sysdep_modify_pen (int pen,
                 unsigned char red, unsigned char green, unsigned char blue)
{
   (*x_modify_pen_func) (pen, red, green, blue);
}

void sysdep_update_display (void)
{
   int new_video_mode = x11_video_mode;

   if (keyboard_key_pressed (KEYCODE_LALT))
   { 
      if (keyboard_key_pressed_memory (KEYCODE_INSERT))
         new_video_mode = X11_WINDOW;
      if (keyboard_key_pressed_memory (KEYCODE_HOME) && xf86_dga_available)
         new_video_mode = X11_DGA;
   }

   if (new_video_mode != x11_video_mode)
   {
      (*x_close_display_func) ();
      x11_video_mode = new_video_mode;
      if (sysdep_create_display () != OSD_OK)
      {
         osd_close_display();   /* This cleans up and must be called to restore
                                   the videomode with dga */
         osd_exit();
         fprintf (stderr_file, "Fatal Error: couldn't create new display while switching display modes\n");
         exit (1);              /* ugly, anyone know a better way ? */
      }
      sysdep_alloc_palette ();
      sysdep_mark_palette_dirty ();
      memset((void *)&xkey[0], FALSE, 128*sizeof(unsigned char) );
      osd_mark_dirty (0, 0, bitmap->width - 1, bitmap->height - 1, 1);
      /* poll mouse twice to clear internal vars */
      if (use_mouse)
      {
         sysdep_mouse_poll ();
         sysdep_mouse_poll ();
      }
   }

   (*x_update_display_func) ();
}

/* these aren't nescesarry under x11 since we have both a graphics window and
   a textwindow (xterm) */
int sysdep_set_video_mode (void)
{
   return OSD_OK;
}

void sysdep_set_text_mode (void)
{
}

#endif /* ifdef x11 */
