/*
 * X-Mame x11 input code
 *
 */
#include "xmame.h"
#include "devices.h"
#include "x11.h"
#include "xkeyboard.h"

static int current_mouse_x = 0;
static int current_mouse_y = 0;

/*
 * Parse keyboard events
 */
void sysdep_update_keyboard (void)
{
  XEvent 		E;
  int	 		keycode,code;
  int			mask;
  int 			*pt;
  static int		old_grab_mouse = FALSE;
  
  /* handle winkey mappings */
  if (use_winkeys)
  {
    extended_code_table[XK_Meta_L&0x1FF] = KEY_LWIN; 
    extended_code_table[XK_Meta_R&0x1FF] = KEY_RWIN; 
  }

  /* query all events that we have previously requested */
  while ( XPending(display) )
  {
    mask = FALSE;
    
    XNextEvent(display,&E);
/*  fprintf(stderr_file,"Event: %d\n",E.type); */

    /* we don't have to check x11_video_mode or extensions like xil here,
       since our eventmask should make sure that we only get the event's matching
       the current update method */
    switch (E.type)
    {
      /* display events */
#ifdef x11
      case Expose:
  	if ( E.xexpose.count == 0 ) x11_window_refresh_screen();
	break;
#endif
      case FocusIn:
	/* to avoid some meta-keys to get locked when wm iconify xmame, we must
	perform a key reset whenever we retrieve keyboard focus */
	memset((void *)&xkey[0], FALSE, 128*sizeof(unsigned char) );
	if (old_grab_mouse)
	{
            if (!XGrabPointer(display, window, True, 0, GrabModeAsync,
                GrabModeAsync, window, None, CurrentTime))
            {
                if (show_cursor) XDefineCursor(display,window,invisible_cursor);
                x11_grab_mouse = TRUE;
            }
	}
	break;
      case FocusOut:
        if (x11_grab_mouse)
        {
            XUngrabPointer(display, CurrentTime);
            if (show_cursor) XDefineCursor(display,window,normal_cursor);
            x11_grab_mouse = FALSE;
            old_grab_mouse = TRUE;
        }
        break;
      case EnterNotify:
	if (use_private_cmap) XInstallColormap(display,colormap);
	break;	
      case LeaveNotify:
	if (use_private_cmap) XInstallColormap(display,DefaultColormapOfScreen(screen));
	break;	
#ifdef USE_XIL
      case ConfigureNotify:
	update_xil_window_size( E.xconfigure.width, E.xconfigure.height );
	break;
#endif
      /* input events */    
      case MotionNotify:
        current_mouse_x += E.xmotion.x_root;
        current_mouse_y += E.xmotion.y_root;
        break;
      case ButtonPress:
        mask = TRUE;
#ifdef USE_DGA
        /* Some buggy combination of XFree and virge screwup the viewport
           on the first mouseclick */
        if(xf86_dga_first_click) { xf86_dga_first_click = 0; xf86_dga_fix_viewport = 1; }
#endif          
      case ButtonRelease:
        switch(E.xbutton.button)
        {
          case Button1:
            mouse_b1 = mask;
            break;
          case Button2:
            mouse_b2 = mask;
            break;
          case Button3:
            mouse_b3 = mask;
            break;
        }
        break;
      case KeyPress:
        mask = TRUE;
      case KeyRelease:
	keycode = XLookupKeysym ((XKeyEvent *) &E, 0);
	/* fprintf(stderr, "Keyevent key:%04X\n", keycode); */
	/* look which table should be used */
	pt=code_table;
        if ( (keycode & 0xfe00) == 0xfe00 )
        {
          pt=extended_code_table;
	  code=keycode&0x01ff;
	}
	else
	  code=keycode&0x00ff;
	/* if unnasigned key ignore it */
	if ( *(pt+code) ) xkey [ *(pt+code) ] = mask;
	break;
#ifdef X11_JOYSTICK
      /* grrr we can't use case here since the event types for XInput devices
         aren't hardcoded, since we should have caught anything else above,
         just asume it's an XInput event */
      default:
	  process_x11_joy_event(&E);
	  break;
#endif
    } /* switch */
  } /* while */
}

/*
 *  keyboard remapping routine
 *  invoiced in startup code
 *  returns 0-> success 1-> invalid from; 2-> invalid to
 */
int sysdep_mapkey(int from, int to)
{
	/* perform tests */
	/* fprintf(stderr_file,"trying to map %x to%x\n",from,to); */
	if ( (to<0) || (to>127) )return 2;
	if ( (from>=0) && (from<=0x00ff) ) 
			{code_table[from]=to; return 0;}
	if ( (from>=0xff00) && (from<=0xffff) ) 
			{extended_code_table[from&0x00ff]=to; return 0;}
	return 1; 
}


void sysdep_mouse_poll (void)
{
	if(x11_video_mode == X11_DGA)
	{
		mouse_delta_x = current_mouse_x;
		mouse_delta_y = current_mouse_y;
		current_mouse_x = current_mouse_y = 0;
	}
	else
	{
		Window root,child;
		int root_x, root_y, pos_x, pos_y;
		unsigned int keys_buttons;
	
		if (!XQueryPointer(display,window, &root,&child, &root_x,&root_y,
			&pos_x,&pos_y,&keys_buttons) )
		{
			mouse_delta_x = mouse_delta_y = 0;
			return;
		}
	
		if ( x11_grab_mouse )
		{
			XWarpPointer(display, None, window, 0, 0, 0, 0,
				visual_width/2, visual_height/2);
			mouse_delta_x = pos_x - visual_width/2;
			mouse_delta_y = pos_y - visual_height/2;
		}
		else
		{
			mouse_delta_x = pos_x - current_mouse_x;
			mouse_delta_y = pos_y - current_mouse_y;
			current_mouse_x = pos_x;
			current_mouse_y = pos_y;
		}
	}
}
